#include "StdAfx.h"
#include "WasGetMailCount.h"
#include "ConvertUTF.h"
#include "MessengerActor.h"
#include "UtilFunction.h"
#include "GameTalkAPIGlobal.h"


///////////////////////////////////////////////////////////////////////////////
//// CWASGetMailCount //////////////////////////////////////////////////////

CWASGetMailCount::CWASGetMailCount(CWASActor *pActor) : CWASCommand(pActor)
{
    m_eMailError = nmailError_None;
    m_ulPrevMailSN = 0;
    m_totalUnreadMail = 0;
}

CWASGetMailCount::~CWASGetMailCount(void)
{

}

void CWASGetMailCount::ExecuteAsync()
{
	m_eError = wasError_None;
	m_eMailError = nmailError_None;

	m_ulPrevMailSN = 0;
	m_totalUnreadMail = 0;

	CMessengerActor *pMessengerActor = static_cast<CMessengerActor *>(m_pActor->GetMessengerActor());
	if(pMessengerActor == NULL)
	{
		return;
	}

	USES_CONVERSION;
	USES_UTF8_CONVERSION;	
	
	WSTRING strURL = _T("nmail.ncloudbiz.com/ntalk/newmail");

	std::stringstream strStream;
	strStream << W2U8(strURL.c_str());

	strStream << "?mtsSessionId=" << pMessengerActor->GetMessengerTicket();

	OutputDebugStringA(strStream.str().c_str());
	OutputDebugStringA("\n");

	m_httpFacade.session().SetURL( strStream.str().c_str(), NULL, HTTP_SERVICE_HTTP );
	m_httpFacade.session().SetMethod(HTTP_METHOD_GET);

	m_httpFacade.session().SetKeepRawHeader(TRUE);
	m_httpFacade.SendRequest(this);
}

bool CWASGetMailCount::ParseAndEventFire(Json::Value * json)
{
	std::string strJsonResult = json->get("Result","").asString();
	if (strJsonResult != "OK")
	{
		m_eMailError = (NTNMailErrorCode)json->get("code", nmailError_System1).asInt();

		std::wstring errorMsg = Conversion::Utf8ToWide(json->get("Message","").asString());
		::OutputDebugString(L"JSON Result is NOT OK\n");
		::OutputDebugString(errorMsg.c_str());

		//NTALK_LOG->Debug(L"[MAIL] FAIL NMAIL (MailCount) : code [%d], msg=[%s]", m_eMailError, errorMsg.c_str());

		return true;
	}

	m_ulPrevMailSN = json->get("lastMailSN", 0).asInt();				// asInt64()

	m_totalUnreadMail = json->get("totalUnreadMail", 0).asInt();		// asInt64()

	return true;
}


// override in drived class
void CWASGetMailCount::OnWasResponse(trid_t trid,
										 bool bSuccess,
										 int /*nWinINetErrorCode*/, 
										 int /*nHttpResponseCode*/, 
										 LPCSTR /*szWasResCodeInHeader*/, 
										 LPCSTR szWasResCodeInXml,
										 Json::Value * json)
{
	if(bSuccess)
	{
		if (ParseAndEventFire(json) == false)
		{
			m_eError = wasError_Protocol;
		}
	}

	GT_ASSERT(m_pActor);

	//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
	//EnterCriticalSection(pCS);

	CGTObserved<IGTWASActorEvent>::listobserver_t observers;
	m_pActor->GetObservers(&observers);
	CGTObserved<IGTWASActorEvent>::listobserver_t::iterator it = observers.begin();

	for (; it != observers.end(); ++it)
		if((*it)->OnGTWASActor_ResponseMailCount(m_pActor, GetID(), m_eError, m_eMailError, m_ulPrevMailSN, m_totalUnreadMail) == false)
			break;

	//LeaveCriticalSection(pCS);

	if(m_pActor)
		m_pActor->ReleaseCommand(GetID());
}


void CWASGetMailCount::OnRequestTimeout(CHttpFacade * /*pHttpF*/, DWORD /*dwSeconds*/)
{
	m_eError = wasError_Timeout;

	GT_ASSERT(m_pActor);

	//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
	//EnterCriticalSection(pCS);

	CGTObserved<IGTWASActorEvent>::listobserver_t observers;
	m_pActor->GetObservers(&observers);
	CGTObserved<IGTWASActorEvent>::listobserver_t::iterator it = observers.begin();	

	for (; it != observers.end(); ++it)
		if((*it)->OnGTWASActor_ResponseMailCount(m_pActor, GetID(), m_eError, m_eMailError, m_ulPrevMailSN, m_totalUnreadMail) == false)
			break;

	//LeaveCriticalSection(pCS);

	if(m_pActor)
		m_pActor->ReleaseCommand(GetID());
}

//// CWASNoteSetReadStatus //////////////////////////////////////////////////////





///////////////////////////////////////////////////////////////////////////////
//// CWASGetMailPreview //////////////////////////////////////////////////////

CWASGetMailPreview::CWASGetMailPreview(CWASActor *pActor) : CWASCommand(pActor)
{
	m_ulMailSerial = 0;
}

CWASGetMailPreview::~CWASGetMailPreview(void)
{

}

void CWASGetMailPreview::ExecuteAsync()
{
	CMessengerActor *pMessengerActor = static_cast<CMessengerActor *>(m_pActor->GetMessengerActor());
	if(pMessengerActor == NULL)
	{
		return;
	}

	USES_CONVERSION;
	USES_UTF8_CONVERSION;	

	WSTRING strURL = _T("nmail.ncloudbiz.com/ntalk/preview");

	std::stringstream strStream;
	strStream << W2U8(strURL.c_str());

	strStream << "?mailSN=" << m_ulMailSerial;
	strStream << "&mtsSessionId=" << pMessengerActor->GetMessengerTicket();

	m_httpFacade.session().SetURL( strStream.str().c_str(), NULL, HTTP_SERVICE_HTTP );	// m_httpFacade Unicode  ʴ´.
	m_httpFacade.session().SetMethod(HTTP_METHOD_GET);

	m_httpFacade.session().SetKeepRawHeader(TRUE);
	m_httpFacade.SendRequest(this);
}

bool CWASGetMailPreview::ParseAndEventFire(Json::Value * json)
{
	std::string strJsonResult = json->get("Result","").asString();
	if (strJsonResult != "OK")
	{
		::OutputDebugString(L"JSON Result is NOT OK\n");

		//OK ƴҰ  ޽ ޴´.
		std::wstring temp = Conversion::Utf8ToWide(json->get("Message","").asString());
		::OutputDebugString(temp.c_str());
		return false;
	}

	m_strSubject = Conversion::Utf8ToWide( (*json)["mailInfo"].get("subject","unknow").asString() ).c_str();
	m_strContent = Conversion::Utf8ToWide( (*json)["mailInfo"].get("contests","unknow").asString() ).c_str();

	//const Json::Value child_ToList = (*json)["mailInfo"]["from"];
	m_strSenderName = Conversion::Utf8ToWide( (*json)["mailInfo"]["from"].get("name","unknow").asString() ).c_str();
	m_strSenderID = Conversion::Utf8ToWide( (*json)["mailInfo"]["from"].get("email","unknow").asString() ).c_str();

	return true;
}


// override in drived class
void CWASGetMailPreview::OnWasResponse(trid_t trid,
									 bool bSuccess,
									 int /*nWinINetErrorCode*/, 
									 int /*nHttpResponseCode*/, 
									 LPCSTR /*szWasResCodeInHeader*/, 
									 LPCSTR szWasResCodeInXml,
									 Json::Value * json)
{
	if(bSuccess)
	{
		if(ParseAndEventFire(json) == false)
		{
			m_eError = wasError_Protocol;
		}
	}

	GT_ASSERT(m_pActor);

	//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
	//EnterCriticalSection(pCS);

	CGTObserved<IGTWASActorEvent>::listobserver_t observers;
	m_pActor->GetObservers(&observers);
	CGTObserved<IGTWASActorEvent>::listobserver_t::iterator it = observers.begin();

	for (; it != observers.end(); ++it)
		if((*it)->OnGTWASActor_ResponseMailPreview(m_pActor, GetID(), m_eError, m_strSubject.c_str(), m_strContent.c_str(), m_strSenderName.c_str(), m_strSenderID.c_str()) == false)
			break;

	//LeaveCriticalSection(pCS);

	if(m_pActor)
		m_pActor->ReleaseCommand(GetID());
}


void CWASGetMailPreview::OnRequestTimeout(CHttpFacade * /*pHttpF*/, DWORD /*dwSeconds*/)
{
	m_eError = wasError_Timeout;

	GT_ASSERT(m_pActor);

	//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
	//EnterCriticalSection(pCS);

	CGTObserved<IGTWASActorEvent>::listobserver_t observers;
	m_pActor->GetObservers(&observers);
	CGTObserved<IGTWASActorEvent>::listobserver_t::iterator it = observers.begin();	

	for (; it != observers.end(); ++it)
		if((*it)->OnGTWASActor_ResponseMailPreview(m_pActor, GetID(), m_eError, m_strSubject.c_str(), m_strContent.c_str(), m_strSenderName.c_str(), m_strSenderID.c_str()) == false)
			break;

	//LeaveCriticalSection(pCS);

	if(m_pActor)
		m_pActor->ReleaseCommand(GetID());
}

//// CWASGetMailPreview //////////////////////////////////////////////////////